﻿Type=Class
Version=5.9
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private ws As WebSocket 'ignore
	' will hold our page information
	Public page As ABMPage
	' page theme
	Private theme As ABMTheme
	' to access the constants
	Private ABM As ABMaterial 'ignore
	' name of the page, must be the same as the class name (case sensitive!)
	Public Name As String = "UsersPage"

    ' iRecs - the number of rows (records to fetch from MySQL table) to show for each page 
	Private iRecs As Int = 4
	
	' your own variables
	Dim ActiveUserId As Int
	
	' part of the select query.... 
	Dim Filter As String = " WHERE UserActive <> 99 "
	
	Private ABMPageId As String = ""
	
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	' build the local structure IMPORTANT!
	BuildPage
End Sub

Private Sub WebSocket_Connected (WebSocket1 As WebSocket)
	'----------------------MODIFICATION-------------------------------
	ws = WebSocket1
	
	ABMPageId = ABM.GetPageID(page, Name,ws)
	
	Dim session As HttpSession = ABM.GetSession(ws, ABMShared.SessionMaxInactiveIntervalSeconds)
	'----------------------START MODIFICATION 4.00-------------------------------
	If session.IsNew Then
		session.Invalidate
		ABMShared.NavigateToPage(ws, "", "./")
		Return
	End If
	'----------------------END MODIFICATION 4.00-------------------------------
	
	
	'  Set needs auth to true
	ABMShared.NeedsAuthorization = True
		
	'  if not logged in, send user back to home page
	If ABMShared.NeedsAuthorization Then
		If session.GetAttribute2("IsAuthorized", "") = "" Then  '  check for auth status.....
			ABMShared.NavigateToPage(ws, ABMPageId, "../")
			Return
		End If
	End If
	
	ABM.UpdateFromCache(Me, ABMShared.CachedPages, ABMPageId, ws)
	If page.ComesFromPageCache Then
		' when we have a page that is cached it doesn't matter if it comes or not from a new connection we serve the cached version.
		Log("Comes from cache")
		page.Refresh
		page.FinishedLoading
	Else
		If page.WebsocketReconnected Then
			Log("Websocket reconnected")
			' when we have a client that doesn't have the page in cache and it's websocket reconnected and also it's session is new - basically when the client had internet problems and it's session (and also cache) expired before he reconnected so the user has content in the browser but we don't have any on the server. So we need to reload the page.
			' when a client that doesn't have the page in cache and it's websocket reconnected but it's session is not new - when the client had internet problems and when he reconnected it's session was valid but he had no cache for this page we need to reload the page as the user browser has content, reconnected but we have no content in cache
			ABMShared.NavigateToPage (ws, ABMPageId, "./" & page.PageHTMLName)
		Else
			' when the client did not reconnected it doesn't matter if the session was new or not because this is the websockets first connection so no dynamic content in the browser ... we are going to serve the dynamic content...
			Log("Websocket first connection")
			page.Prepare
			ConnectPage
		End If
	End If
	Log("  -- This Users Page ID: "&ABMPageId)
End Sub





Private Sub WebSocket_Disconnected
	Log(" about Disconnected")
End Sub


Sub Page_ParseEvent(Params As Map)
	Dim eventName As String = Params.Get("eventname")
	Dim eventParams() As String = Regex.Split(",",Params.Get("eventparams"))
	If eventName = "beforeunload" Then
		Log("preparing for about url refresh")
		'        ws.session.SetAttribute("ABMNewSession", True)
		ABM.RemoveMeFromCache(ABMShared.CachedPages, ABMPageId)
		
		Return
	End If
	
	'  Very Helpful to log control clicks and discover what params are required - and such !!!!
	Log(" *** Users Page Event name: "&eventName&"  Params: "&Params) ' this is used to see what component was clicked and what the parameters are...
	
	If SubExists(Me, eventName) Then
		Params.Remove("eventname")
		Params.Remove("eventparams")
		Select Case Params.Size
			Case 0
				CallSub(Me, eventName)
			Case 1
				CallSub2(Me, eventName, Params.Get(eventParams(0)))
			Case 2
				If Params.get(eventParams(0)) = "abmistable" Then
					Dim PassedTables As List = ABM.ProcessTablesFromTargetName(Params.get(eventParams(1)))
					CallSub2(Me, eventName, PassedTables)
				Else
					CallSub3(Me, eventName, Params.Get(eventParams(0)), Params.Get(eventParams(1)))
				End If
			Case Else
				' cannot be called diretly, to many param
				CallSub2(Me, eventName, Params)
		End Select
	End If
End Sub

public Sub BuildTheme()
	' start with the base theme defined in ABMShared
	theme.Initialize("pagetheme")
	theme.AddABMTheme(ABMShared.MyTheme)
	
	
	' now add our own specific page themes...
	theme.AddTableTheme("tbltheme")
	theme.Table("tbltheme").ZDepth = ABM.ZDEPTH_1
	
	theme.Table("tbltheme").AddCellTheme("bg")
	theme.Table("tbltheme").Cell("bg").BackColor = ABM.COLOR_BLUEGREY
	theme.Table("tbltheme").Cell("bg").ForeColor = ABM.COLOR_WHITE
	theme.Table("tbltheme").Cell("bg").Align = ABM.TABLECELL_HORIZONTALALIGN_LEFT
	
	theme.Table("tbltheme").AddCellTheme("bgr")
	theme.Table("tbltheme").Cell("bgr").BackColor = ABM.COLOR_BLUEGREY
	theme.Table("tbltheme").Cell("bgr").ForeColor = ABM.COLOR_WHITE
	theme.Table("tbltheme").Cell("bgr").Align = ABM.TABLECELL_HORIZONTALALIGN_RIGHT
	
	theme.Table("tbltheme").AddCellTheme("bgc")
	theme.Table("tbltheme").Cell("bgc").BackColor = ABM.COLOR_BLUEGREY
	theme.Table("tbltheme").Cell("bgc").BackColorIntensity = ABM.INTENSITY_LIGHTEN1

	theme.Table("tbltheme").Cell("bgc").ForeColor = ABM.COLOR_YELLOW
	theme.Table("tbltheme").Cell("bgc").Align = ABM.TABLECELL_HORIZONTALALIGN_CENTER
	theme.Table("tbltheme").Cell("bgc").BorderWidth = 2
	theme.Table("tbltheme").Cell("bgc").BorderColor = ABM.COLOR_BLACK

	theme.Table("tbltheme").AddCellTheme("bgb")
	theme.Table("tbltheme").Cell("bgb").BackColor = ABM.COLOR_BLUEGREY
	theme.Table("tbltheme").Cell("bgb").BackColorIntensity = ABM.INTENSITY_LIGHTEN1

	theme.Table("tbltheme").Cell("bgb").ForeColor = ABM.COLOR_AMBER
	theme.Table("tbltheme").Cell("bgb").Align = ABM.TABLECELL_HORIZONTALALIGN_CENTER
	theme.Table("tbltheme").Cell("bgb").BorderWidth = 2
	theme.Table("tbltheme").Cell("bgb").BorderColor = ABM.COLOR_BLACK


	theme.Table("tbltheme").AddCellTheme("nocolor")
	theme.Table("tbltheme").Cell("nocolor").ActiveBackColor = ABM.COLOR_LIGHTGREEN
	theme.Table("tbltheme").Cell("nocolor").ActiveBackColorIntensity = ABM.INTENSITY_LIGHTEN2
	theme.Table("tbltheme").Cell("nocolor").Align = ABM.TABLECELL_HORIZONTALALIGN_CENTER
'	theme.Table("tbltheme").Cell("nocolor").FontSize = 35
'	theme.Table("tbltheme").Cell("nocolor").Align = ABM.TABLECELL_VERTICALALIGN_TOP
	


	theme.Table("tbltheme").AddCellTheme("positive")
	theme.Table("tbltheme").Cell("positive").BackColor = ABM.COLOR_GREEN
	theme.Table("tbltheme").Cell("positive").BackColorIntensity = ABM.INTENSITY_LIGHTEN3
	theme.Table("tbltheme").Cell("positive").ActiveBackColor = ABM.COLOR_GREEN
	theme.Table("tbltheme").Cell("positive").ActiveBackColorIntensity = ABM.INTENSITY_LIGHTEN1
	'theme.Table("tbltheme").Cell("positive").Align = ABM.TABLECELL_HORIZONTALALIGN_RIGHT
	
	theme.Table("tbltheme").AddCellTheme("negative")
	theme.Table("tbltheme").Cell("negative").BackColor = ABM.COLOR_RED
	theme.Table("tbltheme").Cell("negative").BackColorIntensity = ABM.INTENSITY_LIGHTEN3
	theme.Table("tbltheme").Cell("negative").ActiveBackColor = ABM.COLOR_RED
	theme.Table("tbltheme").Cell("negative").ActiveBackColorIntensity = ABM.INTENSITY_LIGHTEN1
	'theme.Table("tbltheme").Cell("negative").Align = ABM.TABLECELL_HORIZONTALALIGN_RIGHT
	
	theme.Table("tbltheme").AddCellTheme("points")
	theme.Table("tbltheme").Cell("points").ActiveBackColor = ABM.COLOR_BLUEGREY
	theme.Table("tbltheme").Cell("points").ActiveBackColorIntensity = ABM.INTENSITY_LIGHTEN2
	theme.Table("tbltheme").Cell("points").Align = ABM.TABLECELL_HORIZONTALALIGN_RIGHT
	
	theme.Table("tbltheme").AddCellTheme("openedit")
	theme.Table("tbltheme").Cell("openedit").ActiveBackColor = ABM.COLOR_BLUEGREY
	theme.Table("tbltheme").Cell("openedit").ActiveBackColorIntensity = ABM.INTENSITY_LIGHTEN2
	theme.Table("tbltheme").Cell("openedit").Align = ABM.TABLECELL_HORIZONTALALIGN_CENTER

	
	
	' add additional themes specific for this page
	theme.AddButtonTheme("bigblue")
	theme.Button("bigblue").BackColor = ABM.COLOR_LIGHTBLUE

	' add additional themes specific for this page
	theme.AddButtonTheme("btnred")
	theme.Button("btnred").BackColor = ABM.COLOR_RED

	
	' modal sheet
	theme.AddContainerTheme("modalcontent")
	theme.Container("modalcontent").BackColor = ABM.COLOR_WHITE
	
	' modal sheet
	theme.AddContainerTheme("modalfooter")
	theme.Container("modalfooter").BackColor = ABM.COLOR_LIGHTBLUE
	
	
End Sub

public Sub BuildPage()
	' initialize the theme
	BuildTheme
	
	' initialize this page using our theme
'	page.InitializeWithTheme(Name, "/ws/" & AppName & "/" & Name, False, theme)
	
	page.InitializeWithTheme(Name, "/ws/" & ABMShared.AppName & "/" & Name, False, ABMShared.SessionMaxInactiveIntervalSeconds, theme)
	
	page.ShowLoader=True
	page.ShowLoaderType=ABM.LOADER_TYPE_MANUAL ' NEW
	
	ABMShared.BuildNavigationBar(page, "Users", "../images/logo.png", "Users", "", "")
		
	' create the page grid
	page.AddRowsM(1,True,0,0, "").AddCells12(1,"")
	page.AddRows(1,True, "").AddCellsOS(1,0,0,0,10,10,11,"").AddCellsOSMP(1,0,0,0,2,2,1,14,0,0,0,"")
	page.AddRows(4,True, "").AddCells12(1,"")
	
	
	page.BuildGrid 'IMPORTANT once you loaded the complete grid AND before you start adding components
		
	' add a modal sheet template to enter user information
	page.AddModalSheetTemplate(BuildInputSheet)
	' add a modal sheet template for a yes no msgbox
	page.AddModalSheetTemplate(BuildMsgBoxYesNo)
		
	
	
End Sub

Sub ConnectPage
	
	ABMShared.ConnectNavigationBar(page)
	
	
	Dim searchUser As ABMInput
	searchUser.Initialize(page, "searchUser", ABM.INPUT_TEXT, "Search user", False, "lightblue")
	page.CellR(1,1).AddComponent(searchUser)
	
	Dim btnSearch As ABMButton
	btnSearch.InitializeFloating(page, "btnSearch", "mdi-action-search", "")
	page.CellR(0,2).AddComponent(btnSearch)
	
	Dim pagination As ABMPagination
	pagination.Initialize(page, "pagination", 5, True, True, "")
	pagination.SetTotalNumberOfPages(0)
	
	page.CellR(1,1).AddComponent(pagination)
	
	' create a ABMtable to show records
	Dim tblUsers As ABMTable
	' The following are basic table constructs...    
	' create and init a table with ID of "tblUsers"
	tblUsers.Initialize(page, "tblUsers",  False,  False, True, "tbltheme")
	' set the headers (titles) for the table.
	tblUsers.SetHeaders(         Array As String ("ID"  , "Access Type"   , "Person's Name"    , "Login (username)"    , "Password"    ,"Photo", "Active" ,"Open/Edit", "Delete"  ))
	' Set the header themes.  Each header column can have a different theme (left justify, color, etc)
	tblUsers.SetHeaderThemes(    Array As String ("bgc" ,     "bgc"       ,       "bgc"        ,        "bgc"          ,    "bgc"      ,    "bgc"   ,   "bgc",   "bgc",   "bgc"   ))
	' define which columns to show in the table.
	' BIG HINT! - Use column 0 for your primary key and set it's visibilty to false.  You will use the value to lookup, edit and delete records....
	tblUsers.SetColumnVisible(   Array As Boolean(False, True      , True      , True       , True          , True       , True , True , True    ))
	
	
	' Responsive tables adjust the view according to device size.  On a phone, a horizontal table just don't work right.
	' Adjust the browser width to see this responsiveness in action...
	tblUsers.IsResponsive = True
	' creates lines between each row (if desired) - The default is false...
	tblUsers.IsBordered = True



	' set the footer - in our case - total number of users found in query
	tblUsers.SetFooter("Total number of users: 0", 12,"bg")
	
	' add our table to a row / cell
	' NOTE: We are using the CellR method (relative to current row)...
	page.CellR(1,1).AddComponent(tblUsers)
	
	' create the action button to Add New Users
	Dim AddUserbtn As ABMActionButton
	AddUserbtn.Initialize(page, "AddUserbtn", "mdi-content-add", "bigblue","")
	AddUserbtn.MainButton.Size = ABM.BUTTONSIZE_LARGE
	
	' add the action button to page (not a row/cell)
	page.AddActionButton(AddUserbtn)
	
	page.FinishedLoading 'IMPORTANT
	
	page.RestoreNavigationBarPosition
	
	checkpics   ' check if pics are assigned...
	
	' now, load users from our MySQL table....
	LoadUsers(1)
	
	' Make sure to refresh the PAGE!!!
	page.Refresh
	
End Sub


Sub checkpics

	Dim SQL As SQL = DBM.GetSQL
	Dim users As List = DBM.SQLSelect(SQL, "SELECT * FROM users", Null)
	If users.Size > 0 Then
		Dim user As Map = users.Get(0)
        Dim pic As String = user.Get("s1")
        If pic.Length < 2 Then
			Try
			  Dim fn As List = File.ListFiles(File.DirApp & "/www/" & ABMShared.AppName & "/images/dumb")
			  For i = 0 To 8 
			  	  Dim str As String = fn.Get(i)
				  Dim usr As Map = users.Get(i)
				  Dim pk As Int = usr.Get("userid")
				  SQL.ExecNonQuery("Update users Set s1 = '"&str&"' Where userid = "&pk)
			  Next
			
			Catch
			  	
			  Log(" No Such Directory or not enough records exist!")
			  	
			End Try	
			
		Else
			Log(" Pics are populated")			
        End If
	End If
		
	DBM.CloseSQL(SQL)
	
	
End Sub


' clicked on the navigation bar
Sub Page_NavigationbarClicked(Action As String, Value As String)
	page.SaveNavigationBarPosition
	If Action = "Users" Then Return
	
	If Action = "LogOff" Then
		ABMShared.LogOff(page)
		Return
	End If
	ABMShared.NavigateToPage(ws, ABMPageId, Value)
End Sub

Sub Page_SignedOffSocialNetwork(Network As String, Extra As String)
	
	' when a user loguts out - set attributes to "" - forcing a new login next time
	page.ws.Session.SetAttribute("authType", "")
	page.ws.Session.SetAttribute("authName", "")
	page.ws.Session.SetAttribute("IsAuthorized", "")
	page.ws.Session.SetAttribute("UserType", "" )
	page.ws.Session.SetAttribute("UserID", "" )
	ABMShared.NavigateToPage(page.ws, ABMPageId, "../")
End Sub


private Sub LoadUsers(fromPage As Int)

	' Here is where we populate the ABMtable with records from our MySQL table
	' Since we don't want all the records initially (could be hundreds), we will use pagination to limit what is fetched from the server
	' Use the query to order your table if you desire

	' Get the ABMTable component...
	Dim tblUsers As ABMTable = page.Component("tblUsers")  ' get the table defined in ConnetPage...
	
	' create a SQL cursor
	Dim SQL As SQL = DBM.GetSQL
	
	' get number of users (select count query) - used in table footer and for pagination....
	Dim numusers As Int = DBM.SQLSelectSingleResult(SQL, "SELECT Count(UserId) as IDS FROM users " & Filter)
	
	'**********************************************************************************************************
	'**********************************************************************************************************
	'
	' Get a list of records - LIMIT these by pagination (page number) and number of records to show - iRecs (global var set to 4 currently)
	' NOTE: We are selecting "certain" fields from the table - and in a certain order - the same order as title headers
	' tblUsers.SetHeaders( Array As String ("ID"  , "Access Type"   , "Person's Name"    , "Login (username)"    , "Password"    ,"Photo", "Active" ,"Open/Edit", "Delete"  ))
	'
	'**********************************************************************************************************
	'**********************************************************************************************************
	Dim qry As String =  "SELECT UserID, UserType,UserName,UserLogin,UserPassword,s1,UserActive  FROM users "& Filter & " LIMIT " & ((fromPage - 1) * iRecs) & ", "&iRecs
	Dim users As List = DBM.SQLSelect(SQL, qry , Null)
	If users.Size = 0 And fromPage > 1 Then
		' we are on a page without any lines (maybe removed by other user?)
		DBM.CloseSQL(SQL)
		fromPage = fromPage - 1
		LoadUsers(fromPage)
		Return
	End If
	
	' if your search returns no results - reset filter and try again...
	If users.Size = 0 And Filter.Length > 3 Then

		Dim searchUser As ABMInput = page.Component("searchUser")
		searchUser.Text = ""
		searchUser.Refresh
		page.ShowToast("srch1","toastgreen"," No Records Found! Click Search Again...",2000,False)
	End If
	Log(" Search was  Found: "&qry)
	
	tblUsers.SetFooter("Total number of users: " & numusers, 12,"bgc")
	
	' clear any old stuff in the table so we may add new...
	tblUsers.Clear
	
	
	For i = 0 To users.Size - 1
	
		' our list is a map of field names  and values
		' we create a new map for each list item
		Dim tblFields As Map = users.Get(i)
		'
		Dim rCellValues As List
		Dim rCellThemes As List
		rCellValues.Initialize
		rCellThemes.Initialize

		'**********************************************************************************************************
		'**********************************************************************************************************
		'**********************************************************************************************************
		' In this example, we shall add ABMTable fields by users.field name...
		' It does not matter in what order we add items now since our query defined the order based on our headers...
		'
		'**********************************************************************************************************
		'**********************************************************************************************************

		For j = 0 To tblFields.Size - 1  ' set the var - Value of map...
			
			' This "var" is used for each "GetKeyAt" that follows....
			Dim var As String
			var = 	tblFields.GetValueAt(j) 
			If var = "null" Then             ' init as string to avoid null_pointer error
				var = ""
			End If

			If tblFields.GetKeyAt(j) = "usertype" Then       '  if users.field name = "this name" then.....
				Dim ut As Int =  tblFields.Get("usertype")
				Select Case ut
					Case 0
						rCellValues.Add("Guest")
					Case 1
						rCellValues.Add("Admin")
					Case 2
						rCellValues.Add("User")
				End Select
				rCellThemes.Add("nocolor")
				Continue
			End If
			

			If tblFields.GetKeyAt(j) = "userid" Then
				rCellValues.Add(var)
				rCellThemes.Add("nocolor")
				Continue
			End If
			
				
			If tblFields.GetKeyAt(j) = "username" Then
				Dim pd As ABMLabel
				pd.Initialize(page,"usrn"&j, var ,ABM.SIZE_H5,False,"")
				rCellValues.Add(pd)
				rCellThemes.Add("nocolor")
				Continue
			End If

			If tblFields.GetKeyAt(j) = "userlogin" Then
				Dim pd As ABMLabel
				pd.Initialize(page,"usrl"&j,var,ABM.SIZE_PARAGRAPH,False,"")
				rCellValues.Add(pd)
				rCellThemes.Add("nocolor")
				Continue
			End If

			'  These are our person pictures...
			' Make sure you have a folder "dumb" under "images" - which contain the images supplied with this lesson
			' The table was initially populated using the method "checkpics" found above...
			'  Notice how some images are round and larger than others....
			' the "var" contains the image name...
			If tblFields.GetKeyAt(j) = "s1" Then
					Try
						Dim img As ABMImage
						img.Initialize(page , "img"&j ,"../images/dumb/"&var, 1.0)

						If i Mod 2 = 0 Then
						   img.IsCircular = True
						   img.SetFixedSize( 150,130)
						   
					    Else
						   img.SetFixedSize( 100,80)
						   img.IsCircular = False
						End If	
							   
						rCellValues.Add(img)
						rCellThemes.Add("nocolor")
					Catch
						rCellValues.Add("No Photo Assigned")
						rCellThemes.Add("nocolor")
					End Try
					Continue
			End If


			If tblFields.GetKeyAt(j) = "userpassword" Then
				Dim pd As ABMLabel
				pd.Initialize(page,"usrp"&j,var,ABM.SIZE_PARAGRAPH,False,"")
				rCellValues.Add(pd)
				rCellThemes.Add("nocolor")
				Continue
			End If


			If tblFields.GetKeyAt(j) = "useractive" Then
				Dim sw As ABMSwitch
				Dim st As Boolean = (var > 0)
				sw.Initialize(  page,"usrsw"&j,  "On", "Off",  st,"switch")
				sw.Enabled = False
				rCellValues.Add(sw)
				rCellThemes.Add("nocolor")
				Continue
			End If


		Next
		
		' add our edit and delete buttons to each row of the table
		' the edit button is blue (by default - no theme)
		
		Dim btnEdit As ABMButton
		btnEdit.InitializeFloating(page, "btnEdit", "mdi-action-visibility", "")
		
		' TIP: uncomment the line below and watch the height of each row shrink.  Use small buttons for more rows per page when required
		' btnEdit.Size = ABM.BUTTONSIZE_SMALL
		
		rCellValues.Add(btnEdit)
		rCellThemes.Add("openedit")

		' the delete button is red (uses a theme - btnred).  Make the button size small - just for the heck of it...
		Dim btnDelete As ABMButton
		btnDelete.InitializeFloating(page, "btnDelete", "mdi-action-delete",  "btnred")
		'btnDelete.Size = ABM.BUTTONSIZE_SMALL
		rCellValues.Add( btnDelete)
		rCellThemes.Add( "openedit")
		
		' Now add a ABMTable row with the list items.  This list must match the number of headers you defined for this abmtable!
		tblUsers.AddRow("uid_"&i,  rCellValues)

		' make sure you have as many items in rCellThemes as in rCellValues!  Must follow IMMEDIATELY AFTER AddRow!
		tblUsers.SetRowThemes( rCellThemes)
	Next

	' refresh the table!!!
	tblUsers.Refresh
	
	DBM.CloseSQL(SQL)
	
	
	' Here we are setting the pagination component.
	' It determines how many pages we have - and which page is currently in view.
	Dim pagination As ABMPagination = page.Component("pagination")
	
	If (numusers Mod iRecs > 0) Or (numusers = 0) Then
		numusers = numusers/iRecs + 1
	Else
		numusers = numusers/iRecs
	End If

	
	pagination.SetTotalNumberOfPages(numusers)
	pagination.SetActivePage(fromPage)
	pagination.Refresh
	
	Log(" Finished loading users")
	
	
End Sub


'private Sub LoadUsers_old(fromPage As Int)  '  from lesson 3......
'
'
'' Here is where we populate the ABMtable with records from our MySQL table
'' Since we don't want all the records initially (could be hundreds), we will use pagination to limit what is fetched from the server
'' Use the query to order your table if you desire 
'
'    ' Get the ABMTable component... 
'	Dim tblUsers As ABMTable = page.Component("tblUsers")  ' get the table defined in ConnetPage...
'	
'	' create a SQL cursor
'	Dim SQL As SQL = DBM.GetSQL
'	
'	' get number of users (select count query) - used in table footer and for pagination....
'	Dim numusers As Int = DBM.SQLSelectSingleResult(SQL, "SELECT Count(UserId) as IDS FROM users " & Filter)  
'	
'	' Get a list of records - LIMIT these by pagination (page number) and number of records to show - iRecs (global var set to 6 currently)
'	Dim qry As String =  "SELECT * FROM users "& Filter & " LIMIT " & ((fromPage - 1) * iRecs) & ", "&iRecs
'	Dim users As List = DBM.SQLSelect(SQL, qry , Null)
'	If users.Size = 0 And fromPage > 1 Then
'		' we are on a page without any lines (maybe removed by other user?)
'		DBM.CloseSQL(SQL)
'		fromPage = fromPage - 1
'		LoadUsers(fromPage)
'		Return
'	End If
'	
'	' if your search returns no results - reset filter and try again...
'	If users.Size = 0 And Filter.Length > 3 Then
'
'		Dim searchUser As ABMInput = page.Component("searchUser")
'		searchUser.Text = ""
'		searchUser.Refresh
'		page.ShowToast("srch1","toastgreen"," No Records Found! Click Search Again...",2000,False)
'	End If
'	Log(" Search was  Found: "&qry)
'	
'	tblUsers.SetFooter("Total number of users: " & numusers, 12,"bgc")
'	
'	' clear any old stuff in the table so we may add new...
'	tblUsers.Clear
'	
'	
'	For i = 0 To users.Size - 1
'	
'	' our list is a map of field names  and values
'	' we create a new map for each list item
'		Dim tblFields As Map = users.Get(i)
'	' 	
'		Dim rCellValues As List
'		Dim rCellThemes As List
'		rCellValues.Initialize
'		rCellThemes.Initialize
'		
'		
'		' the following is ordered by the manner (order) you created the header columns (ie. ID, usertype, username, userlogin, userpassword).
'		rCellValues.Add(tblFields.Get("userid"))
'		
'		' those magic themes...  
'		' I will let you determine why the selected row back color of the edit / delete buttons is different from the other cells....
'		'
'		rCellThemes.Add("nocolor")
'		
'		Select Case tblFields.Get("usertype")
'			Case "0"
'				rCellValues.Add("Guest")
'			Case "1"
'				rCellValues.Add("Admin")
'			Case "2"
'				rCellValues.Add("User")
'		End Select
'		rCellThemes.Add("nocolor")
'		
'		rCellValues.Add(tblFields.Get("username"))
'		rCellThemes.Add("nocolor")
'		
'		rCellValues.Add(tblFields.Get("userlogin"))
'		rCellThemes.Add("nocolor")
'		
'		rCellValues.Add(tblFields.Get("userpassword"))
'		rCellThemes.Add("nocolor")
'		
'		'******* Start of example ******************************************************
'		' we may also use this method to create ABMtable rows
'		' first we get the value of the map item
'		' then we check the key. If the key = "image", then the value is the name of the image file to get from the disk.
'		' create an ABMImage object and show a picture in that cell...
'		
''			For j = 0 To tblFields.Size - 1
''				
''				Dim var As String
''				var = 	tblFields.GetValueAt(j) ' init as string to avoid null_pointer error
''				If var = "null" Then
''					var = ""
''				End If
''				
'				' is this an image field? If yes, get the pic to display....
''				If tblFields.GetKeyAt(j) = "image" Then
''					Dim img As ABMImage
''					img.Initialize(page , "img_"&j ,"../images/pics/"&var, 1.0)
''					img.SetFixedSize( 80,60)  ' set a fixed size for all images!!!!!
''					img.IsCircular = True   '  make this image show in an oval rather than a square
''					rCellValues.Add(img)
''					rCellThemes.Add("nocolor")
''
''				End If
''			Next
'		'******** End of example *****************************************************
'		
'		
'		' add our edit and delete buttons to each row of the table
'		' the edit button is blue (by default - no theme)
'		Dim btnEdit As ABMButton
'		btnEdit.InitializeFloating(page, "btnEdit", "mdi-action-visibility", "")
'		
'		' TIP: uncomment the line below and watch the height of each row shrink.  Use small buttons for more rows per page when required
'		' btnEdit.Size = ABM.BUTTONSIZE_SMALL
'		
'		rCellValues.Add(btnEdit)
'		rCellThemes.Add("openedit")
'
'		' the delete button is red (uses a theme - btnred).  Make the button size small - just for the heck of it...
'		Dim btnDelete As ABMButton
'		btnDelete.InitializeFloating(page, "btnDelete", "mdi-action-delete",  "btnred")
'		btnDelete.Size = ABM.BUTTONSIZE_SMALL
'		rCellValues.Add( btnDelete)
'		rCellThemes.Add( "openedit")
'		
'		' Now add a ABMTable row with the list items.  This list must match the number of headers you defined for this abmtable!
'		tblUsers.AddRow("uid_"&i,  rCellValues)
'
'		' make sure you have as many items in rCellThemes as in rCellValues!  Must follow IMMEDIATELY AFTER AddRow!
'		tblUsers.SetRowThemes( rCellThemes) 
'	Next
'
'	' refresh the table!!!
'	tblUsers.Refresh
'	
'	DBM.CloseSQL(SQL)
'	
'	
'	' Here we are setting the pagination component. 
'	' It determines how many pages we have - and which page is currently in view.
'	Dim pagination As ABMPagination = page.Component("pagination")
'	
'	If (numusers Mod iRecs > 0) Or (numusers = 0) Then
'		numusers = numusers/iRecs + 1
'	Else
'		numusers = numusers/iRecs
'	End If
'
'	
'	pagination.SetTotalNumberOfPages(numusers)
'	pagination.SetActivePage(fromPage)
'	pagination.Refresh
'	
'	Log(" Finished loading users")
'	
'	
'End Sub

Sub pagination_PageChanged(OldPage As Int, NewPage As Int)
	' do your stuff
	LoadUsers(NewPage)
End Sub

Sub AddUserbtn_Clicked(Target As String, SubTarget As String)
	
	' reset all the values on the form for a NEW user
	' set ActiveUserId = 0 to make a NEW record...
	ActiveUserId = 0
	Dim inp As ABMModalSheet = page.ModalSheet("inp")
	Dim combo1 As ABMCombo = inp.Content.Component("combo1")
	combo1.SetActiveItemId("")
	Dim inpName As ABMInput = inp.Content.Component("inpName")
	inpName.Text = ""
	Dim inpLogin As ABMInput = inp.Content.Component("inpLogin")
	inpLogin.Text = ""
	Dim inpPwd As ABMInput = inp.Content.Component("inpPwd")
	inpPwd.Text = ""
	inp.Refresh
	page.ShowModalSheet("inp")
End Sub

Sub tblUsers_Clicked(PassedRowsAndColumns As List)
	' fill with the active values
	Dim tblCellInfo As ABMTableCell = PassedRowsAndColumns.Get(0)
	Dim tblUsers As ABMTable        = page.Component(tblCellInfo.TableName)
	
	'**********************************************************************************************************
	'**********************************************************************************************************
	'**********************************************************************************************************
	'**********************************************************************************************************
	'      NOTE: ABMLabels won't have values included in the PassedRowsAndColumns list.
	' Therefore, we must use col(0) which contains the primary key and do lookups thereafter....
	' See Delete User below....
	'**********************************************************************************************************
	'**********************************************************************************************************
	'**********************************************************************************************************
     
	 ' use col(0) which is primary key....
	ActiveUserId = tblUsers.GetString(tblCellInfo.Row, 0)
	
	
	' If the EDIT button was clicked...... (col 7)
	If tblCellInfo.Column = 7 Then ' edit
		Dim SQL As SQL = DBM.GetSQL
		
		
		' Get the ABMModalSheet ID ("inp") created in BuildInputSheet()
		Dim inp As ABMModalSheet = page.ModalSheet("inp")
		
		' Column 0 of the selected row contains our primary key of the record
		' We will use this to determine what record to Update or Delete!
		' New user additions have ActiveUserId = 0 - so it won't be found in our MySQL table and thus added as a new user
		
		
		Dim users As List = DBM.SQLSelect(SQL, "SELECT * FROM Users WHERE userId=" & ActiveUserId, Null)
		If users.Size > 0 Then
			Dim user As Map = users.Get(0)
			
			
			'  We built this ABMModalSheet earlier - get the contents from each component...
			
			Dim sw As ABMSwitch  = inp.Content.Component("usrsw1")
			sw.State =  (user.Get("useractive") > 0)
			
			Dim combo1 As ABMCombo = inp.Content.Component("combo1")
			combo1.SetActiveItemId("combo1S" & user.Get("usertype"))

			Dim combo2 As ABMCombo = inp.Content.Component("img_S1")
			combo2.SetActiveItemId("" & user.Get("s1"))

			Dim inpName As ABMInput = inp.Content.Component("inpName")
			inpName.Text =  user.Get("username")
		
			Dim inpLogin As ABMInput = inp.Content.Component("inpLogin")
			inpLogin.Text =  user.Get("userlogin")
		
			Dim inpPwd As ABMInput = inp.Content.Component("inpPwd")
			inpPwd.Text =  user.Get("userpassword")
			
			Dim inpimg As ABMImage = inp.Content.Component("img1")
			Try
			inpimg.Source = "../images/dumb/"& user.Get("s1")
			Catch
				Log(" No Such Image")
			End Try	
		End If
		
		DBM.CloseSQL(SQL)
		
		inp.Refresh
		
		' show this modal sheet....
		page.ShowModalSheet("inp")
		
	End If
	
	' If the DELETE button was clicked.... (col 8)
	If tblCellInfo.Column = 8 Then ' delete

		'**********************************************************************************************************
		'**********************************************************************************************************
		'**********************************************************************************************************
		'**********************************************************************************************************
		'      NOTE: ABMLabels won't have values included in the PassedRowsAndColumns list.
		' Therefore, we must use col(0) - simple var - which contains the primary key and do lookups with it....
		'**********************************************************************************************************
		'**********************************************************************************************************
		'**********************************************************************************************************

		Dim usr As String = GetUser(ActiveUserId)
		page.Msgbox2("DELUSR", "This WON'T Delete The User...", " WAIT!"&CRLF&" Delete This User: "&usr& "?", "OK","Cancel",False,ABM.MSGBOX_TYPE_QUESTION,False,ABM.MSGBOX_POS_TOP_CENTER,"")
	End If

End Sub

Sub GetUser( usr As Int) As String   '  lookup the user name for delete....
	
	Dim ret As String = "Not Found"
	
	Dim SQL As SQL = DBM.GetSQL
	Dim users As List = DBM.SQLSelect(SQL, "SELECT * FROM users WHERE UserID = "&usr, Null)
	
	If users.Size > 0 Then
		Dim user As Map = users.Get(0)
	    ret = user.Get("username")
	End If
		
	DBM.CloseSQL(SQL)
    
	Return ret
	
End Sub

Sub img_S1_clicked(target As String)   ' this will change the photo when another image is selected during editing....

	' call your new version of this sheet anything you want - just as long as it refers to the build name: 	= page.ModalSheet("inp")
	Dim ABMGentiInsp_TypeModal As ABMModalSheet = page.ModalSheet("inp")
	Dim ABMGentiInsp_Typeimg  As ABMCombo = ABMGentiInsp_TypeModal.Content.Component("img_S1")
	Dim itm As String = ABMGentiInsp_Typeimg.GetActiveItemId

	ABMGentiInsp_TypeModal.Content.Cell(3,2).RemoveAllComponents
	' since we removed the cell contents - we must init a new one with the SAME id....  ("img1")
	Dim img As ABMImage
	img.Initialize(page,  "img1", "../images/dumb/"&itm, 1.0)
	img.SetFixedSize(100,80)
	
	ABMGentiInsp_TypeModal.Content.Cell(3,2).AddComponent(img)
	ABMGentiInsp_TypeModal.Content.Cell(3,2).Refresh
	
End Sub


Sub BuildInputSheet() As ABMModalSheet
	Dim inp As ABMModalSheet
	inp.Initialize(page, "inp",  False,  False,"")
	inp.IsDismissible = False
	
	inp.Content.AddRowsM(5, True,0,0, "").AddCellsOSMP(2,0,0, 0,6,6,6, 0, 10,0, 0,"cnter") '  AddCells12(1, "")
	inp.Content.BuildGrid

	' ***************************************************************************************
	' ***************************************************************************************
	' NOTE:  We are using CellR from this point on...  Remember what it does!!!...
	' ***************************************************************************************
	' ***************************************************************************************
	
	inp.Content.CellR(0,1).AddComponent(ABMShared.BuildParagraphBQWithZDepth(page,"par1x1","Enter the information for this user.") )
	
	Dim sw As ABMSwitch
	Dim stw As Boolean = True'  (var > 0)
	sw.Initialize(  page,"usrsw1",  "Active", "Not-Active",  stw, "switch")
	sw.Enabled = False
	'sw.Title = "Active"
	inp.Content.CellR(0,2).AddComponent(sw)
	
	
	Dim combo1 As ABMCombo
	combo1.Initialize(page, "combo1", "User Type", 650 , "lightblue")
	combo1.IconName = "mdi-social-group"
	' add items
	combo1.AddItem("combo1S1", "Admin", BuildSimpleItem("S1", "mdi-action-account-circle", "{NBSP}{NBSP}Admin"))
	combo1.AddItem("combo1S2", "User", BuildSimpleItem("S2", "mdi-action-account-circle", "{NBSP}{NBSP}User"))
	combo1.AddItem("combo1S0", "Guest", BuildSimpleItem("S3", "mdi-action-dashboard", "{NBSP}{NBSP}Guest"))
	
	inp.Content.CellR(1,1).AddComponent(combo1)
	
	
	' get all images from our "dumb" folder and make a list...
	Dim lst As List
	lst.Initialize
	Dim lst As List = File.ListFiles(File.DirApp & "/www/" & ABMShared.AppName & "/images/dumb")

	Dim ABMGentiInsp_Typeimg As ABMCombo
	ABMGentiInsp_Typeimg.Initialize(page, "img_S1", "Select a Person",  200, "")
	
	' use icons to prefix our inputs...
	ABMGentiInsp_Typeimg.IconName = "mdi-image-collections"
	For i = 0 To lst.Size - 1
		Dim st As String = lst.Get(i)
		If  st.Contains(".jpg") Then
			'  add these items when they are jpeg files...
			ABMGentiInsp_Typeimg.AddItem( ""&lst.Get(i), ""&lst.Get(i), BuildItem("S"&i,  "../images/dumb/"&lst.Get(i),  "Person", ""&lst.Get(i)))
		End If
	Next
	inp.Content.CellR(0,2).AddComponent(ABMGentiInsp_Typeimg)

	Dim inpName As ABMInput
	inpName.Initialize(page, "inpName", ABM.INPUT_TEXT, "Name", False, "lightblue")
	inpName.IconName = "mdi-action-account-circle"
	inp.Content.CellR(1,1).AddComponent(inpName)

	Dim inpimg As ABMImage
	inpimg.Initialize(page,"img1","",1.0)
	inpimg.SetFixedSize(100,80)
	inp.Content.CellR(0,2).AddComponent(inpimg)
	
	Dim inpLogin As ABMInput
	inpLogin.Initialize(page, "inpLogin", ABM.INPUT_TEXT, "Login", False, "lightblue")
	inpLogin.IconName = "mdi-action-assignment-ind"
	inp.Content.CellR(1,1).AddComponent(inpLogin)
	
	Dim inpPwd As ABMInput
	inpPwd.Initialize(page, "inpPwd", ABM.INPUT_TEXT, "Password", False, "lightblue")
	inpPwd.IconName = "mdi-action-lock-outline"

	inp.Content.CellR(0,2).AddComponent(inpPwd)
	
	inp.Footer.AddRowsM(1,True,0,0, "").AddCells12(1,"")
	inp.Footer.BuildGrid 'IMPORTANT once you loaded the complete grid AND before you start adding components
		
	' create the buttons for the footer, create in opposite order as aligned right in a footer
	Dim CancelBtn As ABMButton
	CancelBtn.InitializeFlat(page, "CancelBtn", "", "", "CANCEL", "transparent")
	inp.Footer.Cell(1,1).AddComponent(CancelBtn)
	
	Dim SaveBtn As ABMButton
	SaveBtn.InitializeFlat(page, "SaveBtn", "", "", "SAVE", "transparent")
	inp.Footer.Cell(1,1).AddComponent(SaveBtn)
		
	Return inp
End Sub


Sub BuildItem(id As String, image As String, Title As String, Subtitle As String) As ABMContainer
	
	' containers are a world onto themselves in ABM land...
	
	Dim ItemCont As ABMContainer
	ItemCont.Initialize(page, id, "")
	ItemCont.AddRowsM(1,False,6,0, "").AddCellsOSMP(1,0,0,0,3,2,2,0,0,16,0,"").AddCellsOS(1,0,0,0,9,10,10,"")
	ItemCont.BuildGrid 'IMPORTANT once you loaded the complete grid AND before you start adding components
	
	Dim SubItemCont As ABMContainer
	SubItemCont.Initialize(page, id & "SubItemCont", "")
	SubItemCont.AddRowsM(1,False,  0,0,"").AddCells12MP(1, -6,0,140,0,"").AddCells12MP(1, 0,0,140,0,"")
	SubItemCont.BuildGrid 'IMPORTANT once you loaded the complete grid AND before you start adding components
	
	ItemCont.Cell(1,2).AddComponent(SubItemCont)
	
	Dim img As ABMImage
	img.Initialize(page, id & "img", image, 1)
	img.SetFixedSize(120,75)
'	img.IsCircular = True
	img.IsResponsive = False
	
	ItemCont.Cell(1,1).AddComponent(img)
	
	Dim lbl1 As ABMLabel
	lbl1.Initialize(page, id & "lbl1", Title, ABM.SIZE_H6, False, "lightblue")
	lbl1.VerticalAlign = True
	
	SubItemCont.Cell(1,1).AddComponent(lbl1)
	
	Dim lbl2 As ABMLabel
	lbl2.Initialize(page, id & "lbl2", Subtitle, ABM.SIZE_H6, False, "")
	lbl2.VerticalAlign = True
	
	SubItemCont.Cell(1,2).AddComponent(lbl2)
	
	Return ItemCont
End Sub


Sub BuildMsgBoxYesNo() As ABMModalSheet
	Dim msg As ABMModalSheet
	msg.Initialize(page, "msg", False, False,"")
	msg.IsDismissible = False
	
	msg.Content.AddRowsM(1, True,0,0, "").AddCells12(1, "")
	msg.Content.BuildGrid
	
	' add paragraph
	msg.Content.CellR(0,1).AddComponent(ABMShared.BuildParagraph(page,"par1","Are you sure you want to delete this user?") )
		
	msg.Footer.AddRowsM(1,True,0,0, "").AddCellsOS(1,6,6,6,3,3,3,"").AddCellsOS(1,0,0,0,3,3,3, "")
	msg.Footer.BuildGrid 'IMPORTANT once you loaded the complete grid AND before you start adding components
	
	' create the buttons for the footer
	Dim msgyes As ABMButton
	msgyes.InitializeFlat(page, "msgyes", "mdi-navigation-check", ABM.ICONALIGN_LEFT, "YES", "transparent")
	msg.Footer.Cell(1,1).AddComponent(msgyes)
	
	Dim msgno As ABMButton
	msgno.InitializeFlat(page, "msgno", "mdi-navigation-close", ABM.ICONALIGN_LEFT, "NO", "transparent")
	msg.Footer.Cell(1,2).AddComponent(msgno)
	
	Return msg
End Sub

Sub BuildSimpleItem(id As String, icon As String, Title As String) As ABMLabel
	Dim lbl As ABMLabel
	If icon <> "" Then
		lbl.Initialize(page, id, Title, ABM.SIZE_H6, True, "header")
	Else
		lbl.Initialize(page, id, Title, ABM.SIZE_H6, True, "")
	End If
	lbl.VerticalAlign = True
	lbl.IconName = icon
	Return lbl
End Sub

Sub SaveBtn_Clicked(Target As String)
	Dim inp As ABMModalSheet = page.ModalSheet("inp")
	
	Dim CurrentFields, WhereFields As Map
	CurrentFields.Initialize
	WhereFields.Initialize
	
	' get the current values
	Dim combo1 As ABMCombo = inp.Content.Component("combo1")
	CurrentFields.Put("UserType", ABMShared.Mid(combo1.GetActiveItemId(), 8, 1))


	' get the image name and save it as well....
	' here we are using our spare table field - "s1"...
	' good thing we had one or we would have to modify the table structure...
	'
	Dim combo2 As ABMCombo = inp.Content.Component("img_S1")
	CurrentFields.Put("s1",DBM.SetQuotes( ABMShared.Mid(combo2.GetActiveItemId(), 1, 8 )))

	
	Dim inpName As ABMInput = inp.Content.Component("inpName")
	CurrentFields.Put("UserName", DBM.SetQuotes(inpName.Text))
	
	Dim inpLogin As ABMInput = inp.Content.Component("inpLogin")
	CurrentFields.Put("UserLogin", DBM.SetQuotes(inpLogin.Text))
	
	Dim inpPwd As ABMInput = inp.Content.Component("inpPwd")
	CurrentFields.Put("UserPassword", DBM.SetQuotes(inpPwd.Text))
	
	CurrentFields.Put("UserActive", 1)  ' we are forcing everyone active.....  
	
	CurrentFields.Put("comp_id", 90)
	CurrentFields.Put("UserRows", 10)
	CurrentFields.Put("Usertz", 1)
	CurrentFields.Put("s2", DBM.SetQuotes(" "))
	CurrentFields.Put("s3", DBM.SetQuotes(" "))
	
	
	
	' build the where values
	WhereFields.Put("UserId", ActiveUserId)
	
	Dim SQL As SQL = DBM.GetSQL
	' When a NEW user is added, the SQL_Select query 
	Dim SQL_Select As String = "SELECT UserID FROM users WHERE UserID = " & ActiveUserId
	Dim SQL_Insert As String = DBM.BuildInsertQuery("users", CurrentFields)
	Dim SQL_Update As String = DBM.BuildUpdateQuery("users", CurrentFields, WhereFields)
	DBM.SQLInsertOrUpdate(SQL, SQL_Select, SQL_Insert, SQL_Update)
	
	DBM.CloseSQL(SQL)
	
	' reload the table
	Dim pagination As ABMPagination = page.Component("pagination")
	LoadUsers(pagination.GetActivePage())
	
	page.CloseModalSheet("inp")
End Sub

Sub CancelBtn_Clicked(Target As String)
	page.CloseModalSheet("inp")
End Sub

Sub msgyes_Clicked(Target As String)
	Dim Currentfields, Wherefields As Map
	Currentfields.Initialize
	Wherefields.Initialize
	
	Currentfields.Put("UserActive", 99)
		
	' build the where values
	Wherefields.Put("UserId", ActiveUserId)
	
	Dim SQL As SQL = DBM.GetSQL
	
	' we're not actual deleting it, as we would also loose his information, so we update his active status
	Dim SQL_Update As String = DBM.BuildUpdateQuery("Users", Currentfields, Wherefields)
	DBM.SQLUpdate(SQL, SQL_Update)
	
	' this would actualy delete him, but then we will have to remove all his cases, attachments, etc also
	'DBUtils.DeleteRecord(DBM.SQLite, "Users", Wherefields)
	
	DBM.CloseSQL(SQL)
	
	' reload the table
	Dim pagination As ABMPagination = page.Component("pagination")
	
	' load the page we were on....
	LoadUsers(pagination.GetActivePage())
	
	page.CloseModalSheet("msg")
End Sub

Sub msgno_Clicked(Target As String)
	page.CloseModalSheet("msg")
End Sub

Sub btnSearch_Clicked(Target As String)
	DoSearch
End Sub

Sub searchUser_EnterPressed()
	DoSearch
End Sub

Sub DoSearch()
	Dim searchUser As ABMInput = page.Component("searchUser")
	Filter = searchUser.Text
	If Filter <> "" Then
		Filter = " WHERE UserActive <> 99 AND (userLogin LIKE '%" & Filter & "%' OR userName LIKE '%" & Filter & "%') "
	Else
		Filter = " WHERE UserActive <> 99 "
	End If
	' reload the table
	LoadUsers(1)
End Sub